package de.hdm_stuttgart.mi.sd1.aufgabe2;

/**
 * Darstellung von Verkaufsartikeln.
 *
 * <p>Ein Artikel benötigt zur Erzeugung drei Angaben: Die Bezeichnung, eine Artikelnummer sowie den Preis in
 *    Euro-Cent (230,50€ sind beispielsweise 23050 Cent). Beispiel:</p>
 *
 *    <table class="goikTableDefaults">
 *        <tr>
 *            <th>Code</th>
 *            <th>Gewünschtes Verhalten</th>
 *        </tr>
 *        <tr>
 *            <td>
 *                <pre>final Artikel a =
 *new Artikel("Fahrrad", 3321, 23050);
 *
 *System.out.println(a);</pre>
 *            </td>
 *            <td>
 *                <pre>Fahrrad, Artikelnummer 3321 zu 230,50€</pre>
 *            </td>
 *        </tr>
 *    </table>
 *
 *    <p>Zwei Artikel Instanzen sind genau dann gleich, wenn ihre Artikelnummern übereinstimmen. Bezeichnung und Preis
 *       sind in Bezug auf einen Vergleich hingegen irrelevant:</p>
 *
 *    <table class="goikTableDefaults">
 *        <tr>
 *            <th>Code</th>
 *            <th>Gewünschtes Verhalten</th>
 *        </tr>
 *        <tr>
 *            <td>
 *                <pre>final Artikel
 *   a = new Artikel("Ball",    <span style="color: green">9132</span>, 4450),
 *   b = new Artikel("Fußball", <span style="color: green">9132</span>, 4350),
 *   c = new Artikel("Fahrrad", <span style="color: red">3321</span>, 23050);
 *
 *System.out.println("Ball / Fußball: " + a.equals(b));
 *System.out.println("Ball / Fahrrad: " + a.equals(c));</pre>
 *            </td>
 *            <td>
 *                <pre>Ball / Fußball: <span style="color: green">true</span>
 *Ball / Fahrrad: <span style="color: red">false</span></pre>
 *            </td>
 *        </tr>
 *    </table>
 *
 *    <section class="implementationHints">
 *        <h3>Hinweise:</h3>
 *        <ul>
 *            <li>Implementieren Sie alle bislang fehlenden Methoden.</li>
 *
 *            <li>Die Unit Tests prüfen neben der formalen Korrektheit auch die Sinnhaftigkeit der Implementierung:
 *             Eine Hashfunktion sollte für verschiedene Objekte »meistens« unterschiedliche Werte zurückliefern.</li>
 *        </ul>
 *    </section>
 */
public class Artikel {
  /**
   * Die Bezeichung eines Artikels, z.B. »Fußball«
   */
  public final String artikelBezeichnung;

  /**
   * Die Artikelnummer.
   */
  final int artikelNummer;

  /**
   * Preisangabe in Cent
   */
  public final int preis;

  /**
   * Konstruktor für Artikelinstanzen.
   *
   * @param artikelBezeichnung Siehe {@link #artikelBezeichnung}.
   * @param artikelNummer Siehe {@link #artikelNummer}.
   * @param preis Siehe {@link #preis}.
   */
  public Artikel(final String artikelBezeichnung, final int artikelNummer, final int preis) {
    this.artikelBezeichnung = artikelBezeichnung;
    this.artikelNummer = artikelNummer;
    this.preis = preis;
  }

  /**
   * Zwei Artikel sind genau dann gleich, wenn ihre {@link #artikelNummer} identisch ist.
   * @param o Das zu vergleichende Objekt.
   * @return <code>true</code> falls das zu vergleichende Objekt ein {@link Artikel} mit
   * identischem Wert für die {@link #artikelNummer} ist, sonst <code>false</code>.
   */
  @Override
  public boolean equals(Object o) {
    if (o instanceof Artikel) {
      final Artikel artikel = (Artikel) o;
      return artikelNummer == artikel.artikelNummer;
    } else {
      return false;
    }
  }

  /**
   * Berechnung im Einklang mit  {@link #equals(Object)}.
   *
   * @return Siehe {@link Object#hashCode()}
   */
  @Override
  public int hashCode() {

    /*
       Hinweis: Achten Sie bei der Implementierung auf die Qualität erzeugter Hash Werte: Beispielsweise ist eine
       konstante Hash Funktion zwar formal korrekt, führt aber ggf. zu schlechter Performance.

       Die Qualität Ihrer Implementierung wird getestet.
     */
    return artikelNummer;
  }

  /**
   * @return Objektdarstellung als String
   */
  @Override
  public String toString() {
    return artikelBezeichnung + ", Artikelnummer " + artikelNummer + " zu " + euroDarstellung() + "€";
  }

  private String euroDarstellung() {
    if (preis < 100) {
      return "0." + preis;
    } else {
      final StringBuilder ret = new StringBuilder(Integer.toString(preis));
      return ret.insert(ret.length() - 2, ".").toString();
    }
  }
}
